@@ -24,48 +24,58 @@ require Carp;
 # new ARS::form(-form => name, -vui => view, -connection => connection)
 
 sub new {
-  my ($class, $self) = (shift, {});
-  my ($b) = bless($self, $class);
+	my ($class, $self) = (shift, {});
+	my ($b) = bless($self, $class);
+	
+	my ($form, $vui, $connection) =  
+	  ARS::rearrange([FORM,VUI,CONNECTION],@_);
+	
+	$connection->pushMessage(&ARS::AR_RETURN_ERROR,
+				 81000,
+				 "usage: new ARS::form(-form => name, -vui => vui, -connection => connection)\nform and connection parameters are required."
+				)    
+	  if(!defined($form) || !defined($connection));
+	
+	$vui = "Default Admin View" unless defined $vui;
+	
+	$self->{'form'}       = $form;
+	$self->{'connection'} = $connection;
+	$self->{'vui'}        = $vui;
+	my %f = ARS::ars_GetFieldTable($connection->{'ctrl'}, 
+				       $form);
+	
+	$connection->tryCatch();
+	$self->{'fields'}     = \%f;
+	
+	my %rev = reverse %f; # convenient
+	$self->{'fields_rev'} = \%rev;
+	
+	my(%t, %enums);
+	
+	foreach (keys %f) {
+		print "caching field: $_\n" if $self->{'connection'}->{'.debug'};
+		my $fv = ARS::ars_GetField($self->{'connection'}->{'ctrl'},
+					   $self->{'form'},
+					   $f{$_});
+		$connection->tryCatch();
+		$t{$_} = $fv->{'dataType'};
+		print "\tdatatype: $t{$_}\n" if $self->{'connection'}->{'.debug'};
 
-  my ($form, $vui, $connection) =  
-    ARS::rearrange([FORM,VUI,CONNECTION],@_);
-
-  $connection->pushMessage(&ARS::AR_RETURN_ERROR,
-			   81000,
-			   "usage: new ARS::form(-form => name, -vui => vui, -connection => connection)\nform and connection parameters are required."
-			   )    
-      if(!defined($form) || !defined($connection));
-
-  $vui = "Default Admin View" unless defined $vui;
-
-  $self->{'form'}       = $form;
-  $self->{'connection'} = $connection;
-  $self->{'vui'}        = $vui;
-  my %f = ARS::ars_GetFieldTable($connection->{'ctrl'}, 
-				 $form);
-
-  $connection->tryCatch();
-  $self->{'fields'}     = \%f;
-
-  my %rev = reverse %f; # convenient
-  $self->{'fields_rev'} = \%rev;
-
-  my(%t, %enums);
-
-  foreach (keys %f) {
-    print "caching field: $_\n" if $self->{'connection'}->{'.debug'};
-    my $fv = ARS::ars_GetField($self->{'connection'}->{'ctrl'},
-			       $self->{'form'},
-			       $f{$_});
-    $connection->tryCatch();
-    $t{$_} = $fv->{'dataType'};
-    print "\tdatatype: $t{$_}\n" if $self->{'connection'}->{'.debug'};
-    $enums{$_} = [@{$fv->{'limit'}}] if $fv->{'dataType'} eq "enum";
-  }
-
-  $self->{'fieldtypes'} = \%t;
-  $self->{'fieldEnumValues'} = \%enums;
-  return $b;
+		if ($fv->{'dataType'} eq "enum") {
+			if (exists $fv->{'limit'}->{'enumLimits'}->{'regularList'}) {
+				$enums{$_} = [@{$fv->{'limit'}->{'enumLimits'}->{'regularList'}}];
+			} else {
+				print "Sorry. I'm not sure what to do with non-regularLists of enums.\n";
+				print "(this enum is type \"", keys %{$fv->{'limit'}->{'enumLimits'}}, "\")\n";
+				print "listStyle = ", $fv->{'limit'}->{'enumLimits'}->{'listStyle'}, "\n";
+				die;
+			}
+		}
+	}
+	
+	$self->{'fieldtypes'} = \%t;
+	$self->{'fieldEnumValues'} = \%enums;
+	return $b;
 }
 
 sub DESTROY {
@@ -87,13 +97,14 @@ sub getEnumValues {
 	return undef;
 }
 
-# query(-query => "qualifier", -maxhits => 100)
+# query(-query => "qualifier", -maxhits => 100, -firstretrieve => 0)
 
 sub query {
     my ($this) = shift;
-    my ($query, $maxhits) = ARS::rearrange([QUERY,MAXHITS], @_);
+    my ($query, $maxhits, $firstretr) = ARS::rearrange([QUERY,MAXHITS,FIRSTRETRIEVE], @_);
     $query = "(1 = 1)" unless defined($query);
     $maxhits = 0 unless defined($maxhits);
+    $firstretr = 0 unless defined($firstretr);
     
     if($this->{'connection'}->{'.debug'}) {
 	print "form->query(".$this->{'form'}.", $query, ".$this->{'vui'}.")\n";
@@ -115,7 +126,7 @@ sub query {
     my @matches = ARS::ars_GetListEntry($this->{'connection'}->{'ctrl'},
 					$this->{'form'},
 					$this->{'qualifier'},
-					$maxhits,
+					$maxhits, $firstretr,
 					@sortOrder);
     
     my(@mids, @mdescs);
@@ -349,6 +360,7 @@ sub value2internal {
 				       "usage: form->value2internal(-field => name, -value => value)\nfield parameter is required.") 
 	unless (defined($f));
     
+    return $v unless defined $v;
     my ($t) = $this->getFieldType($f);
     
     print "value2internal($f, $v) type=$t\n" 
@@ -402,6 +414,7 @@ sub internal2value {
 	# enumeration value exceeds the known list of enumerations,
 	# barf.
 	
+	return undef unless defined $v;
 	if(!defined($this->{'fieldEnumValues'}->{$f}) || 
 	   ($#{$this->{'fieldEnumValues'}->{$f}} < $v) ) {
 	    $this->{'connection'}->pushMessage(&ARS::AR_RETURN_ERROR,
