@@ -1,9 +1,9 @@
 /*
-$Header: /cvs/ARSperl/supportrev.h,v 1.10 2001/10/28 03:45:44 jcmurphy Exp $
+$Header: /cvs/ARSperl/supportrev.h,v 1.11 2003/03/28 05:51:56 jcmurphy Exp $
 
-    ARSperl - An ARS v2 - v4 / Perl5 Integration Kit
+    ARSperl - An ARS v2 - v5 / Perl5 Integration Kit
 
-    Copyright (C) 1995,1996,1997,1998,1999
+    Copyright (C) 1995-2003
 	Joel Murphy, jmurphy@acsu.buffalo.edu
         Jeff Murphy, jcmurphy@acsu.buffalo.edu
 
@@ -14,12 +14,11 @@ $Header: /cvs/ARSperl/supportrev.h,v 1.10 2001/10/28 03:45:44 jcmurphy Exp $
     of ARSperl (or the one that accompanies the source distribution of Perl
     itself) for a full description.
  
-    Comments to:  arsperl@smurfland.cit.buffalo.edu
+    Comments to:  arsperl@arsperl.org
                   (this is a *mailing list* and you must be
                    a subscriber before posting)
 
-    Comments to: arsperl@lurch.cit.buffalo.edu
-    Home Page: http://arsinfo.cit.buffalo.edu
+    Home Page: http://www.arsperl.org
 
 
 */
